@@ -15,19 +15,24 @@
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+*   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA            *
 \*************************************************************************/
 
 //--- LOCAL ---
 #include "../config.h"
-#include "dlgaddcity.h"
 #include "ionlistmodel.h"
-#include "logger/streamlogger.h"
-
+#include "dlgaddcity.h"
+#include "dlgtimezone.h"
+#include "ionlistmodel.h"
+#include "pageanimator.h"
 #include "weatherservice.h"
 #include "yawpconfig.h"
+#include "yawpdefines.h"
 #include "utils.h"
 
+#include "logger/streamlogger.h"
+
+
 //--- QT4 ---
 #include <QDir>
 #include <QFileInfo>
@@ -38,30 +43,49 @@
 #include <KColorScheme>
 #include <KFileDialog>
 #include <KIcon>
-#include <kmessagebox.h>
+#include <KMessageBox>
 #include <Plasma/Theme>
 
 //--- number of maximum allowed cities ---
 #define MAXCITIES	50
 
-
-YawpConfigDialog::YawpConfigDialog( KConfigDialog * parent)
-       : QObject(parent),
-         m_bCityModelChanged(false)
+struct YawpConfigDialog::Private
 {
-	m_pConfigDlg = parent;
-	m_pWeatherModel = new WeatherServiceModel(parent);
+	KConfigDialog              * pConfigDlg;
+	WeatherServiceModel        * pWeatherModel;
+	bool                         bCityModelChanged;
+	bool                         bCitySettingsChanged;
+	bool                         bUnitsChanged;
+
+// This is used to map localized theme names to the english names
+	QMap <int, QString>          theme_map;
+	
+	Yawp::Storage              * pStorage;
+};
+
+YawpConfigDialog::YawpConfigDialog( KConfigDialog * parent, Yawp::Storage * pStorage )
+       : QObject(parent),
+         d(new Private)
+{
+	d->pStorage = pStorage;
+	
+	d->bCityModelChanged = false;
+    d->bCitySettingsChanged = false;
+
+	d->pConfigDlg = parent;
+	d->pWeatherModel = new WeatherServiceModel(pStorage, this);
+	d->pWeatherModel->setObjectName("ConfigModel");
 
 // Fill theme map
-	m_theme_map.insert(0, "default");
-	m_theme_map.insert(1, "purple");
-	m_theme_map.insert(2, "green");
-	m_theme_map.insert(3, "black");
-	m_theme_map.insert(4, "blue");
-	m_theme_map.insert(5, "red");
-	m_theme_map.insert(6, "yellow");
-	m_theme_map.insert(7, "funky");
-	m_theme_map.insert(8, "naked");
+	d->theme_map.insert(0, "default");
+	d->theme_map.insert(1, "purple");
+	d->theme_map.insert(2, "green");
+	d->theme_map.insert(3, "black");
+	d->theme_map.insert(4, "blue");
+	d->theme_map.insert(5, "red");
+	d->theme_map.insert(6, "yellow");
+	d->theme_map.insert(7, "funky");
+	d->theme_map.insert(8, "naked");
 
 	QWidget * locationsPage	= new QWidget( parent );
 	QWidget * settingsPage	= new QWidget( parent );
@@ -83,22 +107,24 @@ YawpConfigDialog::YawpConfigDialog( KConfigDialog * parent)
 
 	//--- LOCATION PAGE ---
 	//
-	cityTableView->setModel( m_pWeatherModel );
+	cityTableView->setModel( d->pWeatherModel );
 	cityTableView->setWordWrap( false );
+	cityTableView->setAlternatingRowColors(true);
 	cityTableView->setColumnWidth( 0, 250 );
 	cityTableView->setColumnWidth( 1, 100 );
 	cityTableView->setColumnWidth( 2, 150 );
 
-	connect(bttnAddCity,	SIGNAL(clicked()),			this, SLOT(addCity()));
-	connect(bttnDeleteCity,	SIGNAL(clicked()),			this, SLOT(deleteCity()));
-	connect(cityTableView,  SIGNAL(clicked(const QModelIndex &)),	this, SLOT(locationSelected(const QModelIndex &)));
-	connect(bttnMoveUp,     SIGNAL(clicked()),			this, SLOT(moveSelectedCityUp()));
-	connect(bttnMoveDown,	SIGNAL(clicked()),			this, SLOT(moveSelectedCityDown()));
+	connect(btnAddCity,     SIGNAL(clicked()),     this, SLOT(addCity()));
+	connect(btnDeleteCity,  SIGNAL(clicked()),     this, SLOT(deleteCity()));
+	connect(cityTableView,  SIGNAL(clicked(const QModelIndex &)),   this, SLOT(locationSelected(const QModelIndex &)));
+	connect(btnMoveUp,      SIGNAL(clicked()),     this, SLOT(moveSelectedCityUp()));
+	connect(btnMoveDown,    SIGNAL(clicked()),     this, SLOT(moveSelectedCityDown()));
+	connect(btnSetTimeZone, SIGNAL(clicked()),     this, SLOT(slotSetTimeZone()));
 
-	bttnAddCity->setIcon( KIcon("list-add") );
-	bttnDeleteCity->setIcon( KIcon("list-remove") );
-	bttnMoveUp->setIcon( KIcon("go-up") );
-	bttnMoveDown->setIcon( KIcon("go-down") );
+	btnAddCity->setIcon( KIcon("list-add") );
+	btnDeleteCity->setIcon( KIcon("list-remove") );
+	btnMoveUp->setIcon( KIcon("go-up") );
+	btnMoveDown->setIcon( KIcon("go-down") );
 
 	//--- THEME PAGE ---
 	//
@@ -123,10 +149,15 @@ YawpConfigDialog::YawpConfigDialog( KConfigDialog * parent)
 	comboPressure->insertItem( 1, i18n("Inches HG"),	QVariant(KUnitConversion::InchesOfMercury) );
 	comboPressure->insertItem( 2, i18n("Millibars"),	QVariant(KUnitConversion::Millibar) );
 	comboPressure->insertItem( 3, i18n("Hectopascals"),	QVariant(KUnitConversion::Hectopascal) );
+#if KDE_IS_VERSION(4,4,92)
+	comboPressure->insertItem( 4, i18n("Millimeters HG"),	QVariant(KUnitConversion::MillimetersOfMercury) );
+#else
+	comboPressure->insertItem( 4, i18n("Torr (Millimeter HG)"),	QVariant(KUnitConversion::Torr) );
+#endif
 
 	comboDistance->insertItem( 3, i18n("Kilometers"),	QVariant(KUnitConversion::Kilometer) );
 	comboDistance->insertItem( 4, i18n("Miles"),		QVariant(KUnitConversion::Mile) );
-#else
+#elif KDE_VERSION_MINOR <= 3
 	comboTemperature->insertItem( 0, i18n("Celsius"),	QVariant(WeatherUtils::Celsius) );
 	comboTemperature->insertItem( 1, i18n("Fahrenheit"),	QVariant(WeatherUtils::Fahrenheit) );
 
@@ -147,10 +178,6 @@ YawpConfigDialog::YawpConfigDialog( KConfigDialog * parent)
 	comboPressure->insertItem( 2, i18n("Millibars"),	QVariant(WeatherUtils::Millibars) );
 	comboPressure->insertItem( 3, i18n("Hectopascals"),	QVariant(WeatherUtils::Hectopascals) );
 
-// Visibility in milimeters? Probably bad not common used.
-//	comboDistance->insertItem( 0, i18n("Centimeters"),	QVariant(WeatherUtils::Centimeters) );
-//	comboDistance->insertItem( 1, i18n("Millimeters"),	QVariant(WeatherUtils::Millimeters) );
-//	comboDistance->insertItem( 2, i18n("Inches"),		QVariant(WeatherUtils::Inches) );
 	comboDistance->insertItem( 3, i18n("Kilometers"),	QVariant(WeatherUtils::Kilometers) );
 	comboDistance->insertItem( 4, i18n("Miles"),		QVariant(WeatherUtils::Miles) );
 #endif
@@ -179,38 +206,65 @@ YawpConfigDialog::YawpConfigDialog( KConfigDialog * parent)
 	parent->resize( minSize );
 	parent->setMinimumSize( minSize );
 
-	if( IonListModel::instance()->rowCount() == 0 )
+	if( d->pStorage->ionListModel()->rowCount() == 0 )
 	{
 		KMessageBox::sorry( parent, i18n("Your system does not have any weather-engine. %1 is useless without at least one weatherengine.", QLatin1String(YAWP_NAME)) );
-		bttnAddCity->setEnabled( false );
+		btnAddCity->setEnabled( false );
 	}
 }
 
 YawpConfigDialog::~YawpConfigDialog()
 {
+	delete d->pWeatherModel;
+	delete d;
 }
 
 void
 YawpConfigDialog::copyCities( const WeatherServiceModel * model )
 {
 	if( model )
-		m_pWeatherModel->copyCities( *model );
+		d->pWeatherModel->copyCities( *model );
 	updateLocationButtons();
-	
-	if( m_pWeatherModel->rowCount() > 0 )
+
+	if( d->pWeatherModel->rowCount() > 0 )
 	{
 		cityTableView->resizeColumnToContents(1);
 		cityTableView->resizeColumnToContents(2);
 	}
 }
 
+const WeatherServiceModel *
+YawpConfigDialog::weatherModel() const
+{
+	return d->pWeatherModel;
+}
+
+bool
+YawpConfigDialog::cityModelChanged() const
+{
+	return d->bCityModelChanged;
+}
+
+bool
+YawpConfigDialog::citySettingsChanged() const
+{
+	return d->bCitySettingsChanged;
+}
+
+bool
+YawpConfigDialog::unitsChanged() const
+{
+	return d->bUnitsChanged;
+}
+
+
 void
 YawpConfigDialog::setData( const Yawp::ConfigData * data )
 {
-	if( ! m_pConfigDlg || !data )
+	if( ! d->pConfigDlg || !data )
 		return;
 	int	iIdx;
-
+	
 	//--- SELECT THIS CITY THE USER HAD SELECTED IN YAWP ---
 	cityTableView->setCurrentIndex( cityTableView->model()->index(data->iCityIndex, 0, QModelIndex()) );
 	locationSelected( cityTableView->currentIndex() );
@@ -249,7 +303,7 @@ YawpConfigDialog::setData( const Yawp::ConfigData * data )
 	checkBoxCustomTheme->setCheckState( data->bUseCustomTheme ? Qt::Checked : Qt::Unchecked );
 	changeThemeState( checkBoxCustomTheme->checkState() );
 
-	comboTheme->setCurrentIndex( m_theme_map.key(data->sBackgroundName) );
+	comboTheme->setCurrentIndex( d->theme_map.key(data->sBackgroundName) );
 	checkBoxUseCustomBackground->setCheckState( data->bUseCustomThemeBackground ? Qt::Checked : Qt::Unchecked );
 	editCustomThemeFile->setText(data->sCustomThemeFile);
 
@@ -258,6 +312,10 @@ YawpConfigDialog::setData( const Yawp::ConfigData * data )
 	enableCustomFontColorOptions( checkBoxUseCustomColor->checkState() );
 	comboColor->setColor( data->fontColor );
 	comboLowerColor->setColor( data->lowFontColor );
+	comboShadowsColor->setColor( data->shadowsFontColor );
+
+	//--- TEXT SHADOWS ---
+	checkBoxDisableTextShadows->setCheckState( data->bDisableTextShadows ? Qt::Checked : Qt::Unchecked );
 
 	//--- PANEL PAGE ---
 	//
@@ -280,7 +338,7 @@ YawpConfigDialog::setData( const Yawp::ConfigData * data )
 
 	checkCompactPanelLayout->setChecked( data->bUseCompactPanelLayout );
 	enableCompactPanelLayout( data->bUseCompactPanelLayout );
-	
+
 	if( data->bUseExtendedTooltip )
 		radioExtendedTooltip->setChecked( true );
 	else
@@ -297,7 +355,7 @@ YawpConfigDialog::setData( const Yawp::ConfigData * data )
 void
 YawpConfigDialog::getData( Yawp::ConfigData * data )
 {
-	if( ! m_pConfigDlg || !data )
+	if( ! d->pConfigDlg || !data )
 		return;
 
 	//--- SELECTED CITY INDEX, THIS WILL BE THE CITY WE SHOW IN YAWP AFTER CLOSING THE CONFIG DIALOG ---
@@ -316,13 +374,13 @@ YawpConfigDialog::getData( Yawp::ConfigData * data )
 	int iPressureIdx = comboPressure->currentIndex();
 	int iDistanceIdx = comboDistance->currentIndex();
 
-	m_bUnitsChanged = false;
+	d->bUnitsChanged = false;
 	if ((int)data->temperatureSystem != comboTemperature->itemData( iTempIdx ).toInt() ||
 	    (int)data->speedSystem       != comboSpeed->itemData( iSpeedIdx ).toInt() ||
 	    (int)data->pressureSystem    != comboPressure->itemData( iPressureIdx ).toInt() ||
 	    (int)data->distanceSystem    != comboDistance->itemData( iDistanceIdx ).toInt() )
 	{
-		m_bUnitsChanged = true;
+		d->bUnitsChanged = true;
 	}
 	data->temperatureSystem = (YAWP_TEMPERATURE_UNIT)comboTemperature->itemData( iTempIdx ).toInt();
 	data->speedSystem       = (YAWP_SPEED_UNIT)comboSpeed->itemData( iSpeedIdx ).toInt();
@@ -343,7 +401,7 @@ YawpConfigDialog::getData( Yawp::ConfigData * data )
 
 	//--- THEMENAME ---
 	data->bUseCustomTheme = (checkBoxCustomTheme->checkState() == Qt::Checked ? true : false);
-	data->sBackgroundName = m_theme_map[comboTheme->currentIndex()];
+	data->sBackgroundName = d->theme_map[comboTheme->currentIndex()];
 	data->bUseCustomThemeBackground = (checkBoxUseCustomBackground->checkState() == Qt::Checked ? true : false);
 	data->sCustomThemeFile = editCustomThemeFile->text();
 
@@ -353,8 +411,12 @@ YawpConfigDialog::getData( Yawp::ConfigData * data )
 	{
 		data->fontColor = comboColor->color();
 		data->lowFontColor = comboLowerColor->color();
+		data->shadowsFontColor = comboShadowsColor->color();
 	}
 
+	//--- TEXT SHADOWS ---
+	data->bDisableTextShadows = (checkBoxDisableTextShadows->checkState() == Qt::Checked ? true : false);
+
 	//--- PANEL PAGE ---
 	//
 	//--- current day settings ---
@@ -400,8 +462,10 @@ YawpConfigDialog::enableCustomFontColorOptions(int state)
 
 	label_comboColor->setEnabled(bEnabled);
 	label_comboLowerColor->setEnabled(bEnabled);
+	label_comboShadowsColor->setEnabled(bEnabled);
 	comboColor->setEnabled(bEnabled);
 	comboLowerColor->setEnabled(bEnabled);
+	comboShadowsColor->setEnabled(bEnabled);
 }
 
 void
@@ -425,7 +489,7 @@ YawpConfigDialog::selectCustomThemeFile()
 	QString filename = KFileDialog::getOpenFileName(
 		KUrl(sDir),
 		"*.svg *.svgz|Scalable Vector Graphics",
-		m_pConfigDlg );
+		d->pConfigDlg );
 	if( !filename.isEmpty())
 		editCustomThemeFile->setText(filename);
 }
@@ -458,23 +522,23 @@ YawpConfigDialog::enableExtendedTooltipOptions( bool bEnabled )
 void
 YawpConfigDialog::addCity()
 {
-	DlgAddCity dlg( m_pConfigDlg );
-	dlg.setWindowTitle( i18n("Add City") );
+	DlgAddCity dlg( d->pStorage, d->pConfigDlg );
 	if( dlg.exec() == QDialog::Accepted )
 	{
 		CityWeather cityInfo = dlg.getSelectedCity();
-		int idx = m_pWeatherModel->addCity( cityInfo );
+		int idx = d->pWeatherModel->addCity( cityInfo );
 
 		if( idx >= 0 )
 		{
-			cityTableView->setCurrentIndex( m_pWeatherModel->index(idx, 0, QModelIndex()) );
+			cityTableView->setCurrentIndex( d->pWeatherModel->index(idx, 0, QModelIndex()) );
 			updateLocationButtons();
-			m_bCityModelChanged = true;
+			d->bCitySettingsChanged = true;
+			d->bCityModelChanged = true;
 		}
 		else
 		{
 			QMessageBox::information(
-				m_pConfigDlg,
+				d->pConfigDlg,
 				i18n("Error"),
 				i18n("City %1 already exists in the list and can not be added again.", cityInfo.city()) );
 		}
@@ -488,20 +552,20 @@ YawpConfigDialog::deleteCity()
 	if( !index.isValid() )
 	{
 		KMessageBox::information(
-			m_pConfigDlg,
+			d->pConfigDlg,
 			i18n("No city has been selected to delete."),
-			m_pConfigDlg->windowTitle() );
+			d->pConfigDlg->windowTitle() );
 		return;
 	}
-	const CityWeather * cityInfo = m_pWeatherModel->getCityInfo( index.row() );
+	const CityWeather * cityInfo = d->pWeatherModel->getCityInfo( index.row() );
 	int ret = KMessageBox::questionYesNo(
-		m_pConfigDlg,
+		d->pConfigDlg,
 		i18n("Do you really want to delete city %1?", cityInfo->city()) );
 	if( ret == KMessageBox::Yes )
 	{
-		m_pWeatherModel->removeRows( index.row(), 1 );
+		d->pWeatherModel->removeRows( index.row(), 1 );
 		updateLocationButtons();
-		m_bCityModelChanged = true;
+		d->bCityModelChanged = true;
 	}
 }
 
@@ -517,46 +581,64 @@ YawpConfigDialog::moveSelectedCityDown()
 	moveSelectedCity( +1 );
 }
 
+void
+YawpConfigDialog::slotSetTimeZone()
+{
+	QModelIndex index = cityTableView->currentIndex();
+	CityWeather * cityInfo = d->pWeatherModel->getCityInfo( index.row() );
+	
+	DlgTimeZone dlg(*cityInfo, d->pStorage);
+	if( dlg.exec() == QDialog::Accepted )
+	{
+		cityInfo->setTimeZone( dlg.selectedTimeZone() );
+		if( cityInfo->countryCode().isEmpty() )
+			cityInfo->setCountryCode( cityInfo->timeZone().countryCode() );
+		d->bCitySettingsChanged = true;
+		d->bCityModelChanged = true;
+	}	
+}
+
 void
 YawpConfigDialog::locationSelected( const QModelIndex & index )
 {
-	int iCities = m_pWeatherModel->rowCount();
-	bttnMoveUp->setEnabled( index.row() > 0 && iCities > 1 );
-	bttnMoveDown->setEnabled( index.row()>= 0 && index.row()+1 < iCities );
+	int iCities = d->pWeatherModel->rowCount();
+	btnMoveUp->setEnabled( index.row() > 0 && iCities > 1 );
+	btnMoveDown->setEnabled( index.row()>= 0 && index.row()+1 < iCities );
 }
 
 void
 YawpConfigDialog::initAnimationCombo( QComboBox * pComboBox )
 {
-	pComboBox->insertItem(  0, i18n("None"),			QVariant(PageAnimator::Jump) );
-	pComboBox->insertItem(  1, i18n("Crossfade"),			QVariant(PageAnimator::CrossFade) );
-	pComboBox->insertItem(  2, i18n("Scanline blend"),		QVariant(PageAnimator::ScanlineBlend) );
-	pComboBox->insertItem(  3, i18n("Flip vertically"),		QVariant(PageAnimator::FlipVertically) );
-	pComboBox->insertItem(  4, i18n("Flip horizontally"),		QVariant(PageAnimator::FlipHorizontally) );
+	pComboBox->insertItem(  0, i18n("None"),                    QVariant(PageAnimator::Jump) );
+	pComboBox->insertItem(  1, i18n("Crossfade"),               QVariant(PageAnimator::CrossFade) );
+	pComboBox->insertItem(  2, i18n("Scanline blend"),          QVariant(PageAnimator::ScanlineBlend) );
+	pComboBox->insertItem(  3, i18n("Flip vertically"),         QVariant(PageAnimator::FlipVertically) );
+	pComboBox->insertItem(  4, i18n("Flip horizontally"),       QVariant(PageAnimator::FlipHorizontally) );
 
 	if( pComboBox == comboIconAnimation )
 		return;
 
-	pComboBox->insertItem(  5, i18n("Roll in vertically"),		QVariant(PageAnimator::RollInVertically) );
-	pComboBox->insertItem(  6, i18n("Roll out vertically"),		QVariant(PageAnimator::RollOutVertically) );
-	pComboBox->insertItem(  7, i18n("Roll in horizontally"),	QVariant(PageAnimator::RollInHorizontally) );
-	pComboBox->insertItem(  8, i18n("Roll out horizontally"),	QVariant(PageAnimator::RollOutHorizontally) );
-	pComboBox->insertItem(  9, i18n("Slide in vertically"),		QVariant(PageAnimator::SlideInVertically) );
-	pComboBox->insertItem( 10, i18n("Slide out vertically"),	QVariant(PageAnimator::SlideOutVertically) );
-	pComboBox->insertItem( 11, i18n("Slide in horizontally"),	QVariant(PageAnimator::SlideInHorizontally) );
-	pComboBox->insertItem( 12, i18n("Slide out horizontally"),	QVariant(PageAnimator::SlideOutHorizontally) );
-	pComboBox->insertItem( 13, i18n("Open vertically"),		QVariant(PageAnimator::OpenVertically) );
-	pComboBox->insertItem( 14, i18n("Close vertically"),		QVariant(PageAnimator::CloseVertically) );
-	pComboBox->insertItem( 15, i18n("Open horizontally"),		QVariant(PageAnimator::OpenHorizontally) );
-	pComboBox->insertItem( 16, i18n("Close horizontally"),		QVariant(PageAnimator::CloseHorizontally) );
+	pComboBox->insertItem(  5, i18n("Roll in vertically"),      QVariant(PageAnimator::RollInVertically) );
+	pComboBox->insertItem(  6, i18n("Roll out vertically"),     QVariant(PageAnimator::RollOutVertically) );
+	pComboBox->insertItem(  7, i18n("Roll in horizontally"),    QVariant(PageAnimator::RollInHorizontally) );
+	pComboBox->insertItem(  8, i18n("Roll out horizontally"),   QVariant(PageAnimator::RollOutHorizontally) );
+	pComboBox->insertItem(  9, i18n("Slide in vertically"),     QVariant(PageAnimator::SlideInVertically) );
+	pComboBox->insertItem( 10, i18n("Slide out vertically"),    QVariant(PageAnimator::SlideOutVertically) );
+	pComboBox->insertItem( 11, i18n("Slide in horizontally"),   QVariant(PageAnimator::SlideInHorizontally) );
+	pComboBox->insertItem( 12, i18n("Slide out horizontally"),  QVariant(PageAnimator::SlideOutHorizontally) );
+	pComboBox->insertItem( 13, i18n("Open vertically"),         QVariant(PageAnimator::OpenVertically) );
+	pComboBox->insertItem( 14, i18n("Close vertically"),        QVariant(PageAnimator::CloseVertically) );
+	pComboBox->insertItem( 15, i18n("Open horizontally"),       QVariant(PageAnimator::OpenHorizontally) );
+	pComboBox->insertItem( 16, i18n("Close horizontally"),      QVariant(PageAnimator::CloseHorizontally) );
 }
 
 void
 YawpConfigDialog::updateLocationButtons()
 {
-	int iCities = m_pWeatherModel->rowCount();
-	bttnAddCity->setEnabled( IonListModel::instance()->rowCount() > 0 && iCities < MAXCITIES );
-	bttnDeleteCity->setEnabled( iCities > 0 );
+	int iCities = d->pWeatherModel->rowCount();
+	btnAddCity->setEnabled( d->pStorage->ionListModel()->rowCount() > 0 && iCities < MAXCITIES );
+	btnDeleteCity->setEnabled( iCities > 0 );
+	btnSetTimeZone->setEnabled( iCities > 0 );
 	locationSelected( cityTableView->currentIndex() );
 }
 
@@ -564,11 +646,11 @@ void
 YawpConfigDialog::moveSelectedCity( int offset )
 {
 	QModelIndex index = cityTableView->currentIndex();
-	if( index.isValid() && m_pWeatherModel->moveCity( index.row(), index.row()+offset ) )
+	if( index.isValid() && d->pWeatherModel->moveCity( index.row(), index.row()+offset ) )
 	{
 		index = index.sibling(index.row()+offset, 0);
 		cityTableView->setCurrentIndex( index );
 		locationSelected( index );
-		m_bCityModelChanged = true;
+		d->bCityModelChanged = true;
 	}
 }
