@@ -15,7 +15,7 @@
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+*   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA            *
 \*************************************************************************/
 
 /*	The weatherserviceclass controls all locations and
@@ -30,6 +30,7 @@
 
 //--- LOCAL ---
 #include "countrymap.h"
+#include "ionlistmodel.h"
 #include "weatherdataprocessor.h"
 #include "weatherservice.h"
 #include "utils.h"
@@ -45,13 +46,14 @@
 #include <QVariant>
 
 //--- KDE4 ---
-#include <klocale.h>
+#include <KLocale>
 #include <Solid/Networking>
 
 
 #define COL_CITY           0
 #define COL_PROVIDER       1
 #define COL_UPDATETIME     2
+#define COL_TIMEZONE       3
 
 
 struct tCityData
@@ -67,7 +69,7 @@ struct WeatherServiceModel::Private
 
 	QList<tCityData *>              vLocations;
 
-	Plasma::DataEngine            * pEngine;
+	const Yawp::Storage           * pStorage;
 	WeatherDataProcessor          * pDataProcessor;
 
 	QMutex                          updateMutex;
@@ -91,85 +93,49 @@ struct WeatherServiceModel::Private
 };
 
 
-WeatherServiceModel::WeatherServiceModel( QObject * parent )
+WeatherServiceModel::WeatherServiceModel( const Yawp::Storage * pStorage,
+                                          QObject * parent,
+                                          WeatherDataProcessor * pDataProcessor )
 	: QAbstractTableModel( parent ),
 	  d( new WeatherServiceModel::Private )
 {
 	d->pParent = this;
-	d->pEngine = NULL;
-	d->pDataProcessor     = NULL;
-	d->iUpdateInterval    = 0;
+	d->pStorage = pStorage;
+	d->pDataProcessor = pDataProcessor;
+	d->iUpdateInterval = 0;
 }
 
 WeatherServiceModel::~WeatherServiceModel()
 {
-	setEngine(NULL);
+	dDebug() << "Delete WeatherServiceModel..." << objectName();
 	qDeleteAll( d->vLocations.begin(), d->vLocations.end() );
+	if( d->pDataProcessor )
+		delete d->pDataProcessor;
 	delete d;
 }
 
-bool
-WeatherServiceModel::setEngine( Plasma::DataEngine * engine )
-{
-	QMutexLocker locker( &d->updateMutex );
-
-	if( d->pEngine )
-	{
-		foreach( tCityData * pData, d->vLocations )
-		{
-			QString sSource( d->getSourceString(pData->cityInfo) );
-			d->pEngine->disconnectSource( sSource, this );
-			dDebug() << "Disconnect " << sSource;
-		}
-	}
-
-	d->pEngine = engine;
-	if( d->pEngine )
-	{
-		if( !d->pDataProcessor )
-			d->pDataProcessor = new WeatherDataProcessor();
-
-		foreach( tCityData * pData, d->vLocations )
-		{
-			d->loadCachedValues( pData->cityInfo );
-		}
-	}
-	else
-	{
-		if( d->pDataProcessor )
-		{
-			delete d->pDataProcessor;
-			d->pDataProcessor = NULL;
-			dDebug() << "Delete DataProcessor";
-		}
-	}
-	return true;
-}
-
-Plasma::DataEngine *
-WeatherServiceModel::engine() const
-{
-	return d->pEngine;
-}
-
 bool
 WeatherServiceModel::connectEngine()
 {
 	QMutexLocker locker( &d->updateMutex );
-	if( !d->pEngine || d->iUpdateInterval <= 0 )
+	if( !d->pDataProcessor || d->iUpdateInterval <= 0 )
 		return false;
 
+	dStartFunct();
+	d->pDataProcessor->setRequestTimeZone(true);
+	Plasma::DataEngine * pEngine = d->pStorage->ionListModel()->engine();
 
 	foreach( tCityData * pData, d->vLocations )
 	{
 		if( !pData->bConnected )
 		{
 			QString sSource( d->getSourceString(pData->cityInfo) );
-			d->pEngine->connectSource( sSource, this, d->iUpdateInterval*60*1000 );
+			pEngine->connectSource( sSource, this, d->iUpdateInterval*60*1000 );
 			pData->bConnected = true;
 		}
 	}
 	connect( Solid::Networking::notifier(), SIGNAL(shouldConnect()), this, SLOT(slotCheckSourceDates()) );
+	dEndFunct();
 	return true;
 }
 
@@ -177,29 +143,32 @@ bool
 WeatherServiceModel::disconnectEngine()
 {
 	QMutexLocker locker( &d->updateMutex );
-	if( !d->pEngine )
+	if( !d->pDataProcessor )
 		return false;
 
+	dStartFunct();
 	disconnect( Solid::Networking::notifier(), SIGNAL(shouldConnect()), this, SLOT(slotCheckSourceDates()) );
 	d->delayedUpdateEventTimer.stop();
 
+	Plasma::DataEngine * pEngine = d->pStorage->ionListModel()->engine();
 	foreach( tCityData * pData, d->vLocations )
 	{
 		if( pData->bConnected )
 		{
 			QString sSource( d->getSourceString(pData->cityInfo) );
-			d->pEngine->disconnectSource( sSource, this );
+			pEngine->disconnectSource( sSource, this );
 			pData->bConnected = false;
 		}
 	}
+	dEndFunct();
 	return true;
 }
 
 bool
-WeatherServiceModel::connectEngineManual()
+WeatherServiceModel::reconnectEngine()
 {
 	QMutexLocker locker( &d->updateMutex );
-	if( !d->pEngine || d->iUpdateInterval <= 0 )
+	if( !d->pDataProcessor || d->iUpdateInterval <= 0 )
 		return false;
 	if( d->manualUpdateTimeoutTimer.isActive() )
 		return false;
@@ -225,7 +194,7 @@ int
 WeatherServiceModel::columnCount( const QModelIndex & parent ) const
 {
 	Q_UNUSED(parent)
-	return 3;
+	return 4;
 }
 
 QVariant
@@ -241,6 +210,7 @@ WeatherServiceModel::headerData( int section,
 		case COL_CITY:          var = i18n("Location"); break;
 		case COL_PROVIDER:      var = i18n("Provider"); break;
 		case COL_UPDATETIME:    var = i18n("Last update"); break;
+		case COL_TIMEZONE:      var = i18n("Timezone"); break;
 		}
 		return var;
 	}
@@ -276,12 +246,16 @@ WeatherServiceModel::data( const QModelIndex & index, int role ) const
 			else
 				var = i18n("no update");
 			break;
+			
+		case COL_TIMEZONE:
+			var = (pCity->timeZone().isValid() ? pCity->timeZone().name() : "?");
+			break;
 		}
 	}
 	else if( role == Qt::DecorationRole )
 	{
 		if( index.column() == COL_CITY && !pCity->countryCode().isEmpty() )
-			var = CountryMap::instance()->getPixmapForCountryCode( pCity->countryCode() );
+			var = d->pStorage->countryMap()->getPixmapForCountryCode( pCity->countryCode() );
 	}
 	else if( role == Qt::BackgroundRole )
 	{
@@ -293,6 +267,10 @@ WeatherServiceModel::data( const QModelIndex & index, int role ) const
 			else
 				var = QColor("#CCFF99");
 		}
+		else if( index.column() == COL_TIMEZONE )
+		{
+			var = QColor( pCity->timeZone().isValid() ? "#CCFF99" : "#FFC0CB" );
+		}
 	}
 	return var;
 }
@@ -330,7 +308,7 @@ WeatherServiceModel::addCity( const CityWeather & cityInfo, int index )
 	foreach( const tCityData * pData, d->vLocations )
 	{
 		// compare the two cities
-		if( pData->cityInfo.compare( cityInfo ) )
+		if( pData->cityInfo.isEqual( cityInfo ) )
 		{
 			dDebug() << "City " << cityInfo.city() << cityInfo.provider() << "already exist.";
 			return -1;
@@ -376,6 +354,7 @@ WeatherServiceModel::copyCities( const WeatherServiceModel & other )
 {
 	QMutexLocker locker( &d->updateMutex );
 	dStartFunct();
+	dInfo() << "CopyCities for " << objectName();
 
 	int iIndex = 0;
 	const tCityData * pOtherData = NULL;
@@ -388,12 +367,15 @@ WeatherServiceModel::copyCities( const WeatherServiceModel & other )
 		pOtherData = *itOtherData;
 		pData = NULL;
 
+		dTracing() << "Copy: " << pOtherData->cityInfo.localizedCityString() << pOtherData->cityInfo.timeZone().name();
+		
 		if( itData == d->vLocations.end() )
 		{
 			pData = d->createNewData( pOtherData->cityInfo );
 			pData->cityInfo.setLastUpdate( pOtherData->cityInfo.lastUpdate() );
+			dTracing() << "   Create new city";
 		}
-		else if( !pOtherData->cityInfo.compare( (*itData)->cityInfo ) )
+		else if( !pOtherData->cityInfo.isEqual( (*itData)->cityInfo ) )
 		{
 			int iResultIndex = iIndex+1;
 			QList<tCityData *>::iterator itResult = itData+1;
@@ -404,17 +386,27 @@ WeatherServiceModel::copyCities( const WeatherServiceModel & other )
 
 			if( itResult != d->vLocations.end() )
 			{
+				// remove city from current position to insert city at another position
 				beginRemoveRows( QModelIndex(), iResultIndex, iResultIndex );
 				pData = (*itResult);
+			
+				//--- set timezone ---
+				pData->cityInfo.setTimeZone( pOtherData->cityInfo.timeZone().name() );
 				d->vLocations.erase(itResult);
 				endRemoveRows();
 			}
 			else
 			{
+				// create a new city
 				pData = d->createNewData( pOtherData->cityInfo );
 				pData->cityInfo.setLastUpdate( pOtherData->cityInfo.lastUpdate() );
 			}
 		}
+		else	// city is already on right position
+		{
+			//--- set timezone ---
+			(*itData)->cityInfo.setTimeZone( pOtherData->cityInfo.timeZone().name() );
+		}
 
 		//--- insert the city ---
 		if( pData )
@@ -447,7 +439,7 @@ WeatherServiceModel::copyCities( const WeatherServiceModel & other )
 
 /*  Returns the requested city with the up to date weather informations.
  */
-const CityWeather *
+CityWeather *
 WeatherServiceModel::getCityInfo( int index ) const
 {
 //	dStartFunct();
@@ -460,94 +452,10 @@ WeatherServiceModel::getCityInfo( int index ) const
 	return pCity;
 }
 
-/*	When we do not have a DataEngine, than we do not have the DataProcessor, In this case we need return a defaulf/dummy value.
- */
-YAWP_DISTANCE_UNIT
-WeatherServiceModel::distanceSystem() const
+WeatherDataProcessor *
+WeatherServiceModel::dataProcessor() const
 {
-#if KDE_IS_VERSION(4,3,70)
-	return (d->pDataProcessor ? d->pDataProcessor->distanceSystem() : KUnitConversion::Kilometer);
-#else
-	return (d->pDataProcessor ? d->pDataProcessor->distanceSystem() : WeatherUtils::Kilometers);
-#endif
-}
-
-bool
-WeatherServiceModel::setDistanceSystem( YAWP_DISTANCE_UNIT unitsystem )
-{
-	bool bReturn(false);
-	if( d->pDataProcessor )
-	{
-		d->pDataProcessor->setDistanceSystem( unitsystem );
-		bReturn = true;
-	}
-	return bReturn;
-}
-
-YAWP_PRESSURE_UNIT
-WeatherServiceModel::pressureSystem() const
-{
-#if KDE_IS_VERSION(4,3,70)
-	return (d->pDataProcessor ? d->pDataProcessor->pressureSystem() : KUnitConversion::Kilopascal);
-#else
-	return (d->pDataProcessor ? d->pDataProcessor->pressureSystem() : WeatherUtils::Kilopascals);
-#endif
-}
-
-bool
-WeatherServiceModel::setPressureSystem( YAWP_PRESSURE_UNIT unitsystem )
-{
-	bool bReturn(false);
-	if( d->pDataProcessor )
-	{
-		d->pDataProcessor->setPressureSystem( unitsystem );
-		bReturn = true;
-	}
-	return bReturn;
-}
-
-YAWP_SPEED_UNIT
-WeatherServiceModel::speedSystem() const
-{
-#if KDE_IS_VERSION(4,3,70)
-	return (d->pDataProcessor ? d->pDataProcessor->speedSystem() : KUnitConversion::Beaufort);
-#else
-	return (d->pDataProcessor ? d->pDataProcessor->speedSystem() : WeatherUtils::Beaufort);
-#endif
-}
-
-bool
-WeatherServiceModel::setSpeedSystem( YAWP_SPEED_UNIT unitsystem )
-{
-	bool bReturn(false);
-	if( d->pDataProcessor )
-	{
-		d->pDataProcessor->setSpeedSystem( unitsystem );
-		bReturn = true;
-	}
-	return bReturn;
-}
-
-YAWP_TEMPERATURE_UNIT
-WeatherServiceModel::temperatureSystem() const
-{
-#if KDE_IS_VERSION(4,3,70)
-	return (d->pDataProcessor ? d->pDataProcessor->temperatureSystem() : KUnitConversion::Celsius);
-#else
-	return (d->pDataProcessor ? d->pDataProcessor->temperatureSystem() : WeatherUtils::Celsius);
-#endif
-}
-
-bool
-WeatherServiceModel::setTemperatureSystem( YAWP_TEMPERATURE_UNIT unitsystem )
-{
-	bool bReturn(false);
-	if( d->pDataProcessor )
-	{
-		d->pDataProcessor->setTemperatureSystem( unitsystem );
-		bReturn = true;
-	}
-	return bReturn;
+	return d->pDataProcessor;
 }
 
 void WeatherServiceModel::setUpdateInterval( int minutes ) {d->iUpdateInterval = minutes; }
@@ -565,17 +473,19 @@ WeatherServiceModel::setDetailsPropertyList( const QList<Yawp::DetailsProperty>
 	return bReturn;
 }
 
+
 void
 WeatherServiceModel::dataUpdated( const QString & sourceName, const Plasma::DataEngine::Data & data )
 {
 	if( !d->pDataProcessor )
 		return;
+	
+	dStartFunct() << sourceName << " data received: " << data;
 
 	const QStringList vTokens = sourceName.split("|");
 	if( vTokens.count() < 3 || vTokens.at(1).compare("weather") != 0 )
 		return;
 
-	dStartFunct() << sourceName;
 	QString sExtra;
 	if( vTokens.count() >= 4 )
 		sExtra = vTokens.at(3);
@@ -584,11 +494,16 @@ WeatherServiceModel::dataUpdated( const QString & sourceName, const Plasma::Data
 	QList<tCityData *>::iterator it = d->vLocations.begin();
 	d->findCity( vTokens.at(0), vTokens.at(2), sExtra, iIndex, it );
 
+	bool bUpdateCityInfo = false;
+	if( d->pDataProcessor->updateCountryInfo( (*it)->cityInfo, data ) )
+		bUpdateCityInfo = true;
+	
 	if( it != d->vLocations.end() &&
 	    d->pDataProcessor->updateLocation( (*it)->cityInfo, data ) )
 	{
-		if( d->pDataProcessor->updateCountryInfo( (*it)->cityInfo, data ) )
+		if( bUpdateCityInfo )
 			d->updateFlags |= CityInfoUpdate;
+		
 		d->pDataProcessor->saveData( (*it)->cityInfo, data );
 		(*it)->cityInfo.setLastUpdate( QDateTime::currentDateTime() );
 
@@ -672,7 +587,7 @@ WeatherServiceModel::slotCheckSourceDates()
 	}
 
 	if( bUpdateNeeded )
-		connectEngineManual();
+		reconnectEngine();
 }
 
 
@@ -723,20 +638,15 @@ void
 WeatherServiceModel::Private::loadCachedValues( CityWeather & cityInfo )
 {
 	// if we have an engine, than try to load cached values
-	if( pEngine && pDataProcessor )
+	if( pDataProcessor )
 	{
-		Plasma::DataEngine::Data data = pEngine->query( getSourceString(cityInfo) );
+		Plasma::DataEngine * pEngine = pStorage->ionListModel()->engine();
+		QString sourceString = getSourceString( cityInfo );
+		Plasma::DataEngine::Data data = pEngine->query( sourceString );
+		
 		if( !data.isEmpty() )
-		{
-//			dDebug() << "DataEngine contains already values for "
-//			         << cityInfo.city() << ", " << cityInfo.provider();
 			pDataProcessor->updateLocation( cityInfo, data );
-		}
 		else
-		{
-//			dDebug() << "try to load cached values for "
-//			         << cityInfo.city() << ", " << cityInfo.provider();
 			pDataProcessor->loadData( cityInfo );
-		}
 	}
 }
