@@ -1,3 +1,22 @@
+/***************************************************************************
+ *   Copyright (C) 2007-2008 by Shawn Starr <shawn.starr@rogers.com>       *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the                         *
+ *   Free Software Foundation, Inc.,                                       *
+ *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
+ ***************************************************************************/
+
 /*************************************************************************\
 *   Copyright (C) 2009 by  Ulf Kreißig                                    *
 *   udev@gmx.net                                                          *
@@ -33,6 +52,7 @@
 #include <KLocale>
 #include <KDateTime>
 #include <KSystemTimeZones>
+#include <kdeversion.h>
 
 const QString WundergroundIon::IonName("wunderground");
 const QString WundergroundIon::ActionValidate("validate");
@@ -117,9 +137,13 @@ struct WundergroundIon::Private
 	QMap<QString, ConditionIcons>     vIconConditionMap;
 	QHash<QString, QString>           vCountryStateMap;
 	
-	QHash<KJob *,  XmlServiceData *>    vServiceJobs;	// contains all running jobs
-	QHash<QString, XmlLookupResult *>   vLookupResults;	// key is the location we're looking for
-	QHash<QString, XmlWeatherData *>    vWeatherData;	// key is ServiceId (e.g.: airport:EDDC)
+	QHash<KJob *,  XmlServiceData *>  vServiceJobs;	// contains all running jobs
+	QHash<QString, XmlLookupResult *> vLookupResults;	// key is the location we're looking for
+	QHash<QString, XmlWeatherData *>  vWeatherData;	// key is ServiceId (e.g.: airport:EDDC)
+	
+#if KDE_IS_VERSION(4,5,2)
+	QStringList                       sourcesToReset;
+#endif
 	
 	void setup_readStation( const QString & sLocation,
 	                        const WeatherStationType type,
@@ -149,19 +173,19 @@ WundergroundIon::WundergroundIon( QObject * parent, const QVariantList & args )
 	d->vIconConditionMap["chancetstorms"] = ChanceThunderstormDay;
 	d->vIconConditionMap["clear"] = ClearDay;
 	d->vIconConditionMap["cloudy"] = Overcast;
-    d->vIconConditionMap["flurries"] = Flurries;
-    d->vIconConditionMap["fog"] = Mist;
-    d->vIconConditionMap["hazy"] = Haze;
-    d->vIconConditionMap["mostlycloudy"] = Overcast;
-    d->vIconConditionMap["mostlysunny"] = FewCloudsDay;
-    d->vIconConditionMap["partlycloudy"] = PartlyCloudyDay;
-    d->vIconConditionMap["partlysunny"] = FewCloudsDay;
-    d->vIconConditionMap["rain"] = Rain;
-    d->vIconConditionMap["sleet"] = RainSnow;
-    d->vIconConditionMap["snow"] = Snow;
-    d->vIconConditionMap["sunny"] = ClearDay;
-    d->vIconConditionMap["tstorms"] = Thunderstorm;
-    d->vIconConditionMap["unknown"] = NotAvailable;
+	d->vIconConditionMap["flurries"] = Flurries;
+	d->vIconConditionMap["fog"] = Mist;
+	d->vIconConditionMap["hazy"] = Haze;
+	d->vIconConditionMap["mostlycloudy"] = Overcast;
+	d->vIconConditionMap["mostlysunny"] = FewCloudsDay;
+	d->vIconConditionMap["partlycloudy"] = PartlyCloudyDay;
+	d->vIconConditionMap["partlysunny"] = FewCloudsDay;
+	d->vIconConditionMap["rain"] = Rain;
+	d->vIconConditionMap["sleet"] = RainSnow;
+	d->vIconConditionMap["snow"] = Snow;
+	d->vIconConditionMap["sunny"] = ClearDay;
+	d->vIconConditionMap["tstorms"] = Thunderstorm;
+	d->vIconConditionMap["unknown"] = NotAvailable;
 
 	/*  We use the country/state to get the countrycode to show the flag in yaWP.
 	 *  But some locations uses country-names and/or state-names we do not know.
@@ -213,8 +237,11 @@ WundergroundIon::reset()
 	}
 	d->vWeatherData.clear();
 
-#if KDE_VERSION_MINOR >= 3
 	/**  Triggered when we get initial setup data for ions that provide a list of places */
+#if KDE_IS_VERSION(4,5,2)
+	d->sourcesToReset = sources();
+	updateAllSources();
+#elif KDE_VERSION_MINOR >= 3
 	emit(resetCompleted(this, true));
 #endif
 	dEndFunct();
@@ -782,7 +809,7 @@ WundergroundIon::updateWeatherSource( const XmlWeatherData & data, const QString
 	}
 
 	int iTempSystem(0), iSpeedSystem(0), iDistanceSystem(0), iPressureSystem(0);
-#if (KDE_VERSION_MINOR == 3 && KDE_VERSION_RELEASE >= 70) || KDE_VERSION_MINOR == 4
+#if KDE_IS_VERSION(4,3,70)
 	if( data.measureSystem == KLocale::Metric )
 	{
 		iTempSystem = KUnitConversion::Celsius;
